#include "mesh.h"
#include "matrix.h"
#include "renumbering.h"
#include "assembly_mass.h"
#include "assembly_energy.h"
#include "interpolate.h"
#include "valarray_util.h"
#include "cg.h"
#include "eye.h"
template <typename T>
std::valarray<T> dirichlet (const mesh<T>& Th, const std::valarray<T>& fh) {
	std::valarray<size_t> num;
	std::valarray<T>      M;
	matrix<T>             A;
	renumbering     (Th, num);
	assembly_mass   (Th, num, M);
	assembly_energy (Th, num, A);
        std::valarray<bool> internal (Th.n_vertice());
        for (size_t i = 0; i < Th.n_vertice(); i++)
            internal [i] = Th.is_internal (i);
        std::valarray<T> b = M*std::valarray<T>(fh [internal]);
        std::valarray<T> x (0.0, Th.n_internal_vertice());
	cg (A, x, b, eye(), 1000, 1e-10);
        std::valarray<T> uh (0.0, Th.n_vertice());
	uh [internal] = x;
	return uh;
}
