#include <valarray>
#include <iostream>
using namespace std;

template <typename T> class matrice;
template <typename M> class const_matrice_rang;

class rang {
    public:
	rang(size_t n1 = 0, size_t n2 = 0) : a(n1), b(n2) {}
	rang operator() (rang sous_rang) const { return rang(a+sous_rang.a,a+sous_rang.b); }
	size_t a, b;
};
template <typename M>
class matrice_rang {
    public:
	typedef typename M::valeur_type valeur_type;
	typedef typename M::matrice_type matrice_type;
	template <typename M2>
	matrice_rang (M2& mm, rang xx, rang yy) 
	  : m(mm.data()), r1(mm.rang1(xx)), r2(mm.rang2(yy)) {}
	template<typename M2>
	matrice_rang<M>& operator = (const M2& m2) {
	    for (size_t i = 0; i < nrow(); i++)
	      for (size_t j = 0; j < ncol(); j++)
		m(r1.a+i,r2.a+j) = m2(i,j);
	    return *this;
	}
	valeur_type operator() (size_t i, size_t j) const { return m(r1.a+i, r2.b+j); }
	valeur_type& operator() (size_t i, size_t j) { return m(r1.a+i, r2.b+j); }
	size_t nrow () const { return r1.b-r1.a; }
	size_t ncol () const { return r2.b-r2.a; }
	matrice_type& data() { return m; }
	const matrice_type& data() const { return m; }
	rang rang1 (rang r) const { return r1(r); }
	rang rang2 (rang r) const { return r2(r); }
	const_matrice_rang<matrice_type> operator() (rang s1, rang s2) const { return const_matrice_rang<matrice_type>(*this, r1(s1), r2(s2)); }	
	matrice_rang<matrice_type> operator() (rang s1, rang s2) { return matrice_rang<matrice_type>(*this, r1(s1), r2(s2)); }	
    protected:
	matrice_type& m;
	rang r1, r2;
};
template <typename M>
class const_matrice_rang {
    public:
	typedef typename M::valeur_type valeur_type;
	typedef typename M::matrice_type matrice_type;
	template <typename M2>
	const_matrice_rang (const M2& mm, rang xx, rang yy) 
	  : m(mm.data()), r1(xx), r2(yy) {}
	valeur_type operator() (size_t i, size_t j) const { return m(r1.a+i, r2.b+j); }
	size_t nrow () const { return r1.b-r1.a; }
	size_t ncol () const { return r2.b-r2.a; }
	const matrice_type& data() const { return m; }
	rang rang1 (rang r) const { return r1(r); }
	rang rang2 (rang r) const { return r2(r); }
	const_matrice_rang<matrice_type> operator() (rang s1, rang s2) const { return const_matrice_rang<matrice_type>(*this, r1(s1), r2(s2)); }	
    protected:
	const matrice_type& m;
	rang r1, r2;
};
template <typename Op, typename M1, typename M2>
class matrice_expr {
    public:
	typedef typename M1::valeur_type valeur_type;
	typedef matrice_expr<Op,M1,M2> matrice_type;
	matrice_expr (Op op0, const M1& mm1, const M2& mm2) 
	  : op(op0), m1(mm1), m2(mm2) {}
	valeur_type operator() (size_t i, size_t j) const { 
		return op(m1(i,j), m2(i,j)); }
	size_t nrow () const { return m1.nrow(); }
	size_t ncol () const { return m1.ncol(); }
	const matrice_type& data() const { return *this; }
	rang rang1 (rang r) const { return r; }
	rang rang2 (rang r) const { return r; }
	const_matrice_rang<matrice_type> operator() (rang s1, rang s2) const { return const_matrice_rang<matrice_type>(*this, s1, s2); }	
    protected:
	Op op;
	const M1& m1;
	const M2& m2;
};
template <typename T>
class matrice {
    protected:
    	size_t d1, d2;
    	valarray<T> m;
    public:
	typedef T valeur_type;
	typedef matrice<T> matrice_type;
	matrice (size_t x1, size_t x2) : d1(x1), d2(x2), m(x1*x2) {}
	matrice (const matrice<T>&) { cerr << "copie !" << endl; }
	matrice<T>& operator= (const matrice<T>& a) { cerr << "affectation !" << endl; }
	T operator() (size_t i, size_t j) const { return m[i+d1*j]; }
	T& operator() (size_t i, size_t j) { return m[i+d1*j]; }
	size_t nrow () const { return d1; }
	size_t ncol () const { return d2; }
	matrice_rang<matrice<T> > operator() (rang x, rang y) { return matrice_rang<matrice<T> >(*this, x, y); }	
	const_matrice_rang<matrice<T> > operator() (rang x, rang y) const { return const_matrice_rang<matrice<T> >(*this, x, y); }	
	matrice_type& data() { return *this; }
	const matrice_type& data() const { return *this; }
	rang rang1 (rang r) const { return r; }
	rang rang2 (rang r) const { return r; }
};
template <typename T>
valarray<T>
operator* (const matrice<T>& a, const valarray<T>& x)
{
    valarray<T> y (a.ncol());
    for (size_t i = 0; i < a.nrow(); i++)
      for (size_t j = 0; j < a.ncol(); j++)
	  y[i] += a(i,j)*x[j];
    return y;
}
template <typename M1, typename M2>
matrice_expr<plus<typename M1::valeur_type>, M1, M2>
operator+ (const M1& a, const M2& b)
{
    typedef plus<typename M1::valeur_type> Op;
    return matrice_expr<Op,M1, M2>(Op(),a,b);
}
template <typename M1, typename M2>
matrice_expr<minus<typename M1::valeur_type>, M1, M2>
operator- (const M1& a, const M2& b)
{
    typedef minus<typename M1::valeur_type> Op;
    return matrice_expr<Op,M1, M2>(Op(),a,b);
}
template <typename M1, typename M2>
matrice<typename M1::valeur_type>
operator* (const M1& a, const M2& b)
{
    typedef typename M1::valeur_type T;
    size_t n = a.nrow();
    matrice<T> c (n, n);
    if (n == 1) {
	c(0,0) = a(0,0)*b(0,0);
	return c;
    }
    size_t m = n/2;
    rang r1 (0,m);
    rang r2 (m, n);
    matrice<T> q1 = (a(r1,r1) + a(r2,r2))*(b(r1,r1) + b(r2,r2));
    matrice<T> q2 = (a(r2,r1) + a(r2,r2))*b(r1,r1);
    matrice<T> q3 = a(r1,r1)*(b(r1,r2) + b(r2,r2));
    matrice<T> q4 = a(r2,r2)*(b(r2,r1) - b(r1,r1));
    matrice<T> q5 = (a(r1,r1) + a(r1,r2))*b(r2,r2);
    matrice<T> q6 = (a(r2,r1) - a(r1,r1))*(b(r1,r1) + b(r1,r2));
    matrice<T> q7 = (a(r1,r2) - a(r2,r2))*(b(r2,r1) + b(r2,r2));
    c(r1,r1) = q1 + q4 - q5 + q7;
    c(r2,r1) = q2 + q4;
    c(r1,r2) = q3 + q5;
    c(r2,r2) = q1 + q3 - q2 + q6;
    return c;
}
template <typename T>
matrice<T>
invert (const matrice<T>& a)
{
    matrice<T> c(a.nrow(), a.nrow());
}
int main()
{
    size_t n = 5;
    matrice<double> a(n,n);
    for (size_t i = 0; i < a.nrow(); i++)
      for (size_t j = 0; j < a.ncol(); j++)
	a(i,j) = 1.0/(i+j+2);
    valarray<double> x(1.0, n);
    valarray<double> y = a*x -x;
    for (size_t i = 0; i < n; i++)
	cout << y[i] << endl;
#undef TODO
#ifdef TODO
    // g++ takes a very long time...
    matrice<double> c = a*a;
#endif // TODO
}


