#include <iostream>
#include <cmath>
#include <cstdlib>
using namespace std;

template <typename T> T F(T u) { return 2.0*u*(u+1.0); }
template <typename T> T J(T u) { return (F(u) + sin(u))*F(u); }

// ================================================================
// question 1
// ================================================================
double dF(double u)  { return 4*u+2; }
double d2F(double u) { return 4; }
double dJ(double u)  { return 2*dF(u)*F(u) + F(u)*cos(u) + dF(u)*sin(u); }
double d2J(double u) { return 2*dF(u)*dF(u) + 2*d2F(u)*F(u)
	               + 2*dF(u)*cos(u) - F(u)*sin(u) 
		       + d2F(u)*sin(u); }
double question1 (double u0) {
    cout << "question 1" << endl;
    double epsilon = 1e-7;
    double u = u0, du = 0;
    size_t i = 0;
    cout << "i u J J' J'' du " << endl;
    do {
        cout << i << " " 
	     << u << " " 
	     << J(u) << " "
	     << dJ(u) << " "
	     << d2J(u) << " "
	     << du << " "
	     << endl;
	du = - dJ(u)/d2J(u);
	u += du;
	++i;
    } while (fabs(du) >= epsilon);
    return u;
}
// ================================================================
// question 2
// ================================================================

template <typename T>
class da {
  public:
    da (T u0 = T(0.0)) : _u(u0), _du(0.0), _d2u(0.0) {}
    da (const da<T>& v) : _u(v._u), _du(v._du), _d2u(v._d2u) {}
    da<T>& operator= (const da<T>& v) {
	    _u = v._u; _du = v._du; _d2u = v._d2u; return *this; }
    void is_variable() { _du=T(1.0); }
    T u()   const { return _u; }
    T du()  const { return _du; }
    T d2u() const { return _d2u; }
    void set_u  (T u0)   { _u   = u0; }
    void set_du (T du0)  { _du  = du0; }
    void set_d2u(T d2u0) { _d2u = d2u0; }
    friend da<T> operator+ (da<T> f, da<T> g) {
	 da<T> h;
	 h._u  = f._u    + g._u;
	 h._du = f._du   + g._du;
	 h._d2u = f._d2u + g._d2u;
	 return h;
    }
    friend da<T> operator+ (da<T> f, double g) { return operator+ (f, da<T>(g)); }
    friend da<T> operator* (da<T> f, da<T> g) {
	 da<T> h;
	 h._u   = f._u  * g._u;
	 h._du  = f._du * g._u + f._u * g._du;
	 h._d2u = f._d2u * g._u + 2*f._du * g._du + f._u*g._d2u;
	 return h;
    }
    friend da<T> operator* (double f, da<T> g) { return operator* (da<T>(f), g); }
    friend da<T> sin (da<T> f) {
	 da<T> g;
	 g._u   = sin(f._u);
	 g._du  = f._du*cos(f._u);
	 g._d2u = f._d2u*cos(f._u) - f._du * f._du * sin(f._u);
	 return g;
    }
  protected:
    T _u, _du, _d2u;
};
double question2 (double u0_base) {
    cout << "question 2" << endl;
    double epsilon = 1e-7;
    da<double> u0(u0_base);
    u0.is_variable();
    cout << "i u J J' J'' du " << endl;
    da<double> u = u0;
    double du = 0;
    size_t i = 0;
    do {
        da<double> j = J(u);
        cout << i << " " 
	     << u.u() << " " 
	     << j.u() << " "
	     << j.du() << " "
	     << j.d2u() << " "
	     << du << " "
	     << endl;
	du = - j.du()/j.d2u();
	u.set_u (u.u() + du);
	++i;
    } while (fabs(du) >= epsilon);
    return u.u();
}
// ================================================================
// question 3
// ================================================================

template <typename T>
class daf {
  public:
    daf (T x0 = T(0.0), T d0 = T(0.0)) : _v(x0), _d(d0) {}
    daf (const daf<T>& x) : _v(x._v), _d(x._d) {}
    daf<T>& operator= (const daf<T>& x) { _v = x._v; _d = x._d; return *this; }
    T& value()       { return _v; }
    T value()  const { return _v; }
    T diff()   const { return _d; }
    void is_variable() { _d=T(1.0); }

    friend daf<T> operator+ (daf<T> f, daf<T> g) {
	 daf<T> h;
	 h._v = f._v + g._v;
	 h._d = f._d + g._d;
	 return h;
    }
    friend daf<T> operator* (daf<T> f, daf<T> g) {
	 daf<T> h;
	 h._v = f._v * g._v;
	 h._d = f._d * g._v + f._v * g._d;
	 return h;
    }
    friend daf<T> sin (daf<T> f) {
	 daf<T> g;
	 g._v = sin(f._v);
	 g._d = f._d*cos(f._v);
	 return g;
    }
  // data:
    T _v, _d;
};
template <typename T> T G(T u) { return u*u*u; }
template <typename T> T dG_dx  (T x) { daf<T> y (x); y.is_variable(); return G(y).diff(); }
template <typename T> T d2G_dx2(T x) { daf<T> y (x); y.is_variable(); return dG_dx(y).diff(); }

double question3 (double u0_base, size_t n) {

    daf<double> u(u0_base);
    u.is_variable();
    cout << "G0 = " << G(u).value() << endl;
    cout << "dG0 = " << dG_dx(u).value() << endl;
    cout << "d2G0 = " << d2G_dx2(u).value() << endl;
    return 0;
}
// ================================================================
int main(int argc, char**argv) {
    double u0 = (argc > 1) ? atof(argv[1]) :  -3;
    question1(u0);
    question2(u0);
    question3(u0,0);
    return 0;
}
