#include <valarray>
#include <iostream>
template<typename T>
class sparse_array {
  public:
    explicit sparse_array(size_t n1=0) : idx(), val(), n(n1) {}
    sparse_array(const sparse_array<T>& y) : idx(y.idx), val(y.val), n(y.n) {}
    sparse_array<T>& operator= (const sparse_array<T>& y) {
        idx.operator= (y.idx);
        val.operator= (y.val);
	n = y.n;
	return *this;
    }
    ~sparse_array() {}
    size_t size() const { return n; }
    size_t nnz() const { return idx.size(); }
    sparse_array<T> operator+ (const sparse_array<T>& y) const;
    sparse_array<T> operator- (const sparse_array<T>& y) const;
    template <typename U> 
    friend sparse_array<U> operator* (const U& a, const sparse_array<U>& x);
    template <typename U>
    friend std::istream& operator >> (std::istream&, sparse_array<U>&);
    template <typename U>
    friend std::ostream& operator << (std::ostream&, const sparse_array<U>&);
  protected:
    size_t add_size (const sparse_array<T>& y) const;
    std::valarray<size_t> idx;
    std::valarray<T>      val;
    size_t n;
};
template<typename T>
std::istream&
operator>> (std::istream& is, sparse_array<T>& x) {
    size_t nnz;
    is >> x.n >> nnz;
    x.idx.resize(nnz);
    x.val.resize(nnz);
    for (size_t p = 0; p < nnz; p++) {
        is >> x.idx[p] >> x.val[p];
	x.idx[p]--;
    }
    return is;
}
template<typename T>
std::ostream&
operator<< (std::ostream& os, const sparse_array<T>& x) {
    os << x.size() << " " << x.nnz() << std::endl;
    for (size_t p = 0; p < x.nnz(); p++)
        os << x.idx[p]+1 << " " << x.val[p] << std::endl;
    return os;
}
template <typename T>
sparse_array<T>
operator*(const T& a, const sparse_array<T>& x) {
    sparse_array<T> z;
    z.n = x.n;
    if (a == 0) return z;
    z.idx.resize(x.nnz());
    z.val.resize(x.nnz());
    for (size_t p = 0; p < x.nnz(); p++) {
        z.idx [p] =   x.idx[p];
        z.val [p] = a*x.val[p];
    }
    return z;
}
template<typename T>
size_t
sparse_array<T>::add_size (const sparse_array<T>& y) const {
    size_t p = 0;
    size_t q = 0;
    size_t r = 0;
    while (p != nnz() && q != y.nnz()) {
	if (idx[p] == y.idx[q]) {
	  if (val[p++] + y.val[q++] != 0) r++;
	} else if (idx[p] < y.idx[p]) {
	  p++; r++;
	} else {
	  q++; r++;
        }
    }
    return r + nnz() - p + y.nnz() - q;
}
template<typename T>
sparse_array<T>
sparse_array<T>::operator+ (const sparse_array<T>& y) const {
    sparse_array<T> z;
    size_t nnz_z = add_size (y);
    z.idx.resize(nnz_z);
    z.val.resize(nnz_z);
    z.n = n;
    size_t p = 0;
    size_t q = 0;
    size_t r = 0;
    while (p != nnz() && q != y.nnz()) {
	if (idx[p] == y.idx[q]) {
	  T tmp = val[p] + y.val[q];
	  if (tmp != 0) {
	    z.idx[r] = idx[p];
	    z.val[r] = tmp;
	    r++;
          }
	  p++; q++;
	} else if (idx[p] < y.idx[p]) {
	  z.idx[r] = idx[p];
	  z.val[r] = val[p];
	  r++;
	  p++;
	} else {
	  z.idx[r] = y.idx[q];
	  z.val[r] = y.val[q];
	  r++;
	  q++;
        }
    }
    while (p != nnz()) {
        z.idx[r] = idx[p];
	z.val[r] = val[p];
	r++;
	p++;
    }
    while (q != y.nnz()) {
        z.idx[r] = y.idx[q];
	z.val[r] = y.val[q];
	r++;
	q++;
    }
    return z;
}
using namespace std;
int main() {
    sparse_array<double> x, y;
    cin >> x >> y;
    cout << "x = " << x
         << "y = " << y;
    sparse_array<double> z = x+y;
    cout << "z = " << z
         << "z = " << 2.0*z;
}
