// ---------------------------------------------------------------------------
// array with expression template
//  voir Vandevoorde & Josuttis, p. 328
//  enleve les T pour simplifier : T=double
// ---------------------------------------------------------------------------
#include <iostream> 
#include <functional> // plus, minus, multiplies, divides, modulus, negate
class simple_array {
    public:
     	explicit simple_array(size_t sz = 0, const double& initval = 0)
	  : data(new double[sz]), n(sz) {
	    for (size_t i = 0; i < n; i++) data[i] = initval;
	    std::cerr << "init "<< n << " data : "  << data[0] << std::endl;
	}
     	simple_array(const simple_array& b)
	  : data(new double[b.n]), n(b.n) {
	    for (size_t i = 0; i < n; i++) data[i] = b.data[i];
	    std::cerr << "copy "<< n << " data : "  << data[0] << std::endl;
	}
     	~simple_array() { delete [] data; }
     	simple_array& operator= (simple_array& b) {
	    if (&b == this) return *this;
	    if (n != b.n) {
     	      delete [] data; 
	      data = new double[b.n];
	      n    = b.n;
	    }
	    for (size_t i = 0; i < n; i++) data[i] = b.data[i];
	    std::cerr << "affect "<< n << " data : "  << data[0] << std::endl;
	    return *this;
	}
	size_t size() const { return n; }
	double  operator[] (size_t i) const { return data[i]; }
	double& operator[] (size_t i) { return data[i]; }

	template <typename Op, typename B, typename C>
	friend class expr;
    protected:
	double* data;
	size_t n;
};
template <typename A>
class array_traits {
  public:
    typedef A const& reference_type;
};
template <typename Op, typename A, typename B>
class expr {
    public:
	template <typename A2>
	friend class array;
	
	template <typename Op2, typename A2, typename B2>
	friend class expr;
	
	expr(const A& a1, const B& b1) : op(), a(a1), b(b1) {}
	size_t size() const { return a->size(); }
	double operator[] (size_t i) const { return op(a[i],b[i]); }
    protected:
	Op op;
	typename array_traits<A>::reference_type a;
	typename array_traits<B>::reference_type b;
};
class array_scalar {
    public:
	array_scalar(const double& v) : value(v) {}
	size_t size() const { return 0; }
	double operator[] (size_t) const { return value; }
    protected:
	double const& value;
};
template <>
class array_traits<array_scalar> {
  public:
    typedef array_scalar reference_type;
};
template <typename A = simple_array>
class array {
    public:
	explicit array (size_t n = 0, const double& initval = 0) : a(n,initval) {}
	template <typename B>
	array (const array<B>& b) : a(b.size()) {
	  for (size_t i = 0; i < b.size(); i++) 
	    a[i] = b[i];
        }
	template <typename B>
	array<A>& operator= (const array<B>& expr) {
	  for (size_t i = 0; i < a.size(); i++) 
	    a[i] = expr[i];
	  return *this;
	}
	size_t size() const { return a.size(); }
	double  operator[] (size_t i) const { return a[i]; }
	double& operator[] (size_t i) { return a[i]; }

	template <typename Op, typename B, typename C>
	friend class expr;

	template <typename B>
	friend class array;

	template <typename B>
	const array<class expr<std::plus<double>,A,B> >
	operator+ (const array<B>& y) const {
            return array<expr<std::plus<double>,A,B> >(expr<std::plus<double>,A,B>(a,y.a));
        }
	template <typename B>
	const array<class expr<std::multiplies<double>,A,B> >
	operator* (const array<B>& y) const {
            return array<expr<std::multiplies<double>,A,B> >(expr<std::multiplies<double>,A,B>(a,y.a));
        }
	friend
	const array<class expr<std::multiplies<double>,array_scalar,A> >
	operator* (const double& lambda, const array<A>& x) {
	    array_scalar coef (lambda);
            typedef expr<std::multiplies<double>,array_scalar,A> C;
            return array<C>(C(coef,x.a));
        }
    protected:
	array (const A& a0) : a(a0) {}
	A a;
};
#include <iostream>
using namespace std;
int main () {
  size_t n = 5;
  array<> x(n, 1), y(n, 2), z(n, 5), t(n, 0);
  cerr << "before..." << endl;
  t = 1.2*x + y*z;
  cerr << "after..." << endl;
  for (size_t j = 0; j < z.size(); j++)
    cout << j << " " << t [j] << endl;
}
