#include "matrix.h"
template <typename T>
class ilu : public matrix<T> {
  public:
    ilu(const matrix<T>& a);
    std::valarray<T> solve (const std::valarray<T>&) const;
  protected:
    std::valarray<size_t> diag_ptr;
};
// A = L*U
// L and U stored in the same data struct
// L is unitary; U includes the diag of the data struct
template <typename T>
ilu<T>::ilu(const matrix<T>& a)
: matrix<T>(), diag_ptr(a.nrow())
{
  matrix<T>::resize(a.nrow(),a.ncol(), a.nnz());
  matrix<T>::ptr = a.ptr;
  matrix<T>::idx = a.idx;
  matrix<T>::val = a.val;
  for (size_t i = 0; i < a.nrow(); i++)
    for (size_t p = a.ptr[i]; p < a.ptr[i+1]; p++)
      if (a.idx[p] == i) {
        diag_ptr[i] = p;
	break;
      }
  std::valarray<size_t> iw (size_t(0), a.nrow());
  for (size_t k = 0; k < a.nrow(); k++) {
    for (size_t p = a.ptr[k]; p < a.ptr[k+1]; p++)
      iw[a.idx[p]] = p;
    size_t p = a.ptr[k];
    size_t jrow = size_t(-1);
    for (p = a.ptr[k]; p < a.ptr[k+1]; p++) {
      jrow = a.idx[p];
      if (jrow >= k) break;
      T tl = matrix<T>::val[p]/matrix<T>::val[diag_ptr[jrow]];
      matrix<T>::val[p] = tl;
      for (size_t jj = diag_ptr[jrow]+1; jj < a.ptr[jrow+1]; jj++) {
        size_t jw = iw[a.idx[jj]];
        if (jw != 0)
  	  matrix<T>::val[jw] -= tl * matrix<T>::val[jj];
      }
    }
    diag_ptr[k] = p;
    if (jrow != k || matrix<T>::val[p] == 0) {
      std::cerr << "ilu failed at step " << k << " jrow = " << jrow
	      << " val[" << p << "] = " << matrix<T>::val[p] << std::endl;
      exit (1);
    }
    for (size_t p = a.ptr[k]; p < a.ptr[k+1]; p++)
      iw[a.idx[p]] = 0;
  }
}
template <typename T>
std::valarray<T> ilu<T>::solve (const std::valarray<T>& b) const
{
  // solve : L*y = b
  std::valarray<T> x = b;
  for (size_t i = 0; i < b.size(); i++) {
    for (size_t p = matrix<T>::ptr[i]; p < diag_ptr[i]; p++)
      x[i] -= matrix<T>::val[p] * x[matrix<T>::idx[p]];
  }
  // solve : U*x = y; where U is unitaire
  for (size_t i = b.size(); i > 0; i--) {
    for (size_t p = diag_ptr[i-1]+1; p < matrix<T>::ptr[i]; p++)
      x[matrix<T>::idx[p]] -= matrix<T>::val[p] * x[i-1];
    x[i-1] /= matrix<T>::val[diag_ptr[i-1]];
  }
  return x;
}
