#include <stdlib.h>
#include <math.h>
#include <iostream>
#include <vector>
#include "jacobi.h"
#include "jacobi_gauss_lobatto_deflation.h"
#include "numeric_constant.h"

using namespace std;

template <typename T>
T jacobi_gauss_lobatto_check (unsigned int N) {
        const T pi = numeric_constant<T>::pi();

	unsigned int ngrid = 200;
	T a = -0.5;
	T b = -0.5;
	cout << "# jacobi-gauss-lobatto nodes" << endl
	     << "# a = " << a << endl
	     << "# b = " << b << endl
	     << "# N = " << N << endl;

	for (unsigned int i = 0; i < ngrid+1; i++) {
	    T x = -1 + 2.0*i/(ngrid+1);
	    T p, d, p1, d1, p2, d2; 
	    jacobi (a, b, N, x, p, d, p1, d1, p2, d2);
	    cout << x << "	" << p << endl;
	}
	cout << endl;

	cout << "# i	x	x_exact		x-x_exact	(1-x^2)*P'(x)" << endl;
	vector<T> xgl (N+1);
	jacobi_gauss_lobatto_deflation (a, b, N, xgl);
	T err = 0;
	for (unsigned int i = 0; i < N+1; i++) {
	    T p, d, p1, d1, p2, d2; 
	    jacobi (a, b, N, xgl [i], p, d, p1, d1, p2, d2);
	    T xgl_exact = cos (i*pi/N);
	    cout << "# " 
		 << xgl [i] << "	" 
		 << xgl_exact << "	"
		 << fabs(xgl [i]-xgl_exact) << "	"
		 << d << endl;
	    cout << xgl [i]   << " " << 0 << endl
	         << xgl [i]   << " " << p << endl
		 << endl;
	    err = max (err, fabs(xgl [i]-xgl_exact));
	}
	return err;
}
int main (int argc, char **argv) {
	unsigned int N = argc > 1 ? atoi(argv[1]) : 5;
	cerr << "err " << jacobi_gauss_lobatto_check<double> (N) << endl;
}
