#ifndef _COURS_NUMERIC_CONSTANT_H
#define _COURS_NUMERIC_CONSTANT_H

#define HAVE_LIMITS

#ifdef HAVE_LIMITS
#include <limits>
#else
#include "numeric_limits.h"
#endif // HAVE_LIMITS

template <typename T> struct numeric_constant {};

template <> struct numeric_constant<float> {	
	typedef float value_type;
	static value_type pi ()      { return 3.14159265358979323846; }
	static int digits10 ()       { return std::numeric_limits<value_type>::digits10; }
	static int digits   ()       { return std::numeric_limits<value_type>::digits; }
	static value_type epsilon () { return std::numeric_limits<value_type>::epsilon(); }
};
template <> struct numeric_constant<double> {
	typedef double value_type;
	static value_type pi ()      { return 3.14159265358979323846; }
	static int digits10 ()       { return std::numeric_limits<value_type>::digits10; }
	static int digits   ()       { return std::numeric_limits<value_type>::digits; }
	static value_type epsilon () { return std::numeric_limits<value_type>::epsilon(); }
};
template <> struct numeric_constant<long double> {
	typedef long double value_type;
	static value_type pi ()      { return 3.1415926535897932384626433832795029L; }
	static int digits10 ()       { return std::numeric_limits<value_type>::digits10; }
	static int digits   ()       { return std::numeric_limits<value_type>::digits; }
	static value_type epsilon () { return std::numeric_limits<value_type>::epsilon(); }
};
#ifdef HAVE_GINAC
#include <ginac/ginac.h>
template <> struct numeric_constant<GiNaC::ex> {
	typedef GiNaC::ex value_type;
	static value_type pi ()      { return GiNaC::Pi.evalf (); }
	static int digits10 ()       { return GiNaC::Digits; }
	static value_type epsilon () { return GiNaC::pow (value_type(10.0), value_type(-1.0*GiNaC::Digits)); }
	// pb pour repasser en int:
	// static int digits   ()       { return int(1+GiNaC::log(value_type(10.))/GiNaC::log(value_type(2.))*value_type(1.0*GiNaC::Digits)); }
};
#endif // HAVE_GINAC

#endif // _COURS_NUMERIC_CONSTANT_H
