#!/bin/sh
#
# check Orszag spectral preconditionner
#
# trace the spectra of 
#	the energy matrix A from 1d spectral pb
#	the energy matrix M from 1d F.D. pb at same node
#	the matrix new  inv(M)*A operator for iterative method
#
# --------------------------------------------------------------
# file dependencies:
#
#	spectral_dump_energy_tst : binary, and .cc
#
#	orszag_plot.m		 : octave files
#	read_mm.m
#	read_v.m
#	ps2epsi
# --------------------------------------------------------------
PS2EPSI=ps2ps
prog=$0
prog=`expr $prog : '.*/\(.*\)' \| $prog`
prog=`expr $prog : '\(.*\)\.sh' \| $prog`
if test "$1" = ""; then
    	echo "usage: $prog <N> [<name>=\"a\"]" >&2
    	exit 1
fi
# ------------------------------
# generates the matrices in .mm
# ------------------------------
N=$1
if test "$2" = ""; then
    root="A"
else
    root=$2
fi
name="${root}_${N}"
spectral_dump_energy_tst ${N} ${root}

# ------------------------------
# the octave script
# ------------------------------
cat > ${name}_spectra.m << EOF
	w = read_v  ("${name}_weight.v");
	d = read_mm ("${name}_derivative.mm");
	m = read_mm ("${name}_precond.mm");
	[N,dummy] = size(m); N = N+1;
	a = d(:,2:N)'*diag(w)*d(:,2:N);
	ima = inv(m)*a;
	orszag_plot ("a",   a,   "${name}_energy");
	orszag_plot ("ima", ima, "${name}_result");
EOF
echo "! file \"${name}_spectra.m\" created."

echo "! octave -q < ${name}_spectra.m"
octave -q < ${name}_spectra.m
status=$?
if test $status -ne 0; then
	echo "$prog: octave exits with status = $status" >&2
	exit $status
fi
# ------------------------------
# the gnuplot scripts
# ------------------------------
cat > ${name}_main.iplot << EOF
	# file generated by : ${prog} ${N} ${root}
        xyratio         = 0.125
        border_factor   = 0.2           # adjust me !
        x_bound = pi**2/4  		# = 2.467, the max eigval. for the precond. matrix
        set size ratio xyratio
        set noytics
        set xtics nomirror
        x_border_factor = border_factor*xyratio
        xmax = (1+x_border_factor)*lambda_max
	xmax = (xmax < x_bound) ? x_bound : xmax
        xmin = -x_border_factor*lambda_max
        ymin = -1 - border_factor
        ymax =  1 + border_factor
        f(x) = 2*ymax
        plot [xmin:xmax][ymin:ymax] f(x) notitle w l 2
EOF
# ----------------
cat ${name}_energy.iplot ${name}_main.iplot > ${name}_energy.plot
echo "! file \"${name}_energy.plot\" created."

echo "! gnuplot ${name}_energy.plot"
gnuplot ${name}_energy.plot
${PS2EPSI} ${name}_energy.ps   ${name}_energy.epsi
mv         ${name}_energy.epsi ${name}_energy.ps
# ------------
cat ${name}_result.iplot ${name}_main.iplot > ${name}_result.plot
echo "! file \"${name}_result.plot\" created."

echo "! gnuplot ${name}_result.plot"
gnuplot ${name}_result.plot
${PS2EPSI} ${name}_result.ps   ${name}_result.epsi
mv         ${name}_result.epsi ${name}_result.ps
# ------------------------------
# cleans
# ------------------------------
rm -f ${name}_spectra.m
rm -f ${name}_derivative.mm ${name}_precond.mm ${name}_weight.v
rm -f ${name}_energy.iplot ${name}_precond.iplot ${name}_result.iplot
rm -f ${name}_main.iplot
# rm -f ${name}_energy.plot  ${name}_precond.plot  ${name}_result.plot
