//
//	attention:  dans ]-1,1[ avec n*n elements
//
#include "poisson_2d_sinus.h"
#include "interpolate_2d.icc"
#include "finite_differences_2d_plot.icc"
#include "sqr.h"
#include <iostream>
using namespace std;
double u (const double& x, const double& y) { return (1-sqr(x))*(1-sqr(y))/2; }
double f (const double& x, const double& y) { return 2-sqr(x)-sqr(y); }

int main (int argc, char** argv) {
        size_t n = (argc > 1) ? atoi(argv[1]) : 10;
	double h = 2.0/n;
        valarray<double> pi_h_u = interpolate (n, u);
	valarray<double> pi_h_f = interpolate (n, f);
	valarray<double> b (sqr(n-1));
	index_type elt (n+1);
	index_type unknown (n-1);
        for (size_t i = 0; i < n-1; i++)
            for (size_t j = 0; j < n-1; j++)
	      b [unknown (i,j)] = sqr(h)*pi_h_f [elt (i+1,j+1)];
	poisson_2d_sinus A (n-1);
	valarray<double> x = A.solve (b);
	valarray<double> uh (0.0, sqr(n+1));
        for (size_t i = 0; i < n-1; i++)
            for (size_t j = 0; j < n-1; j++)
	      uh [elt (i+1, j+1)] = x [unknown (i,j)];
	plot (n, uh, "uh");
	plot (n, pi_h_u, "pi_h_u");
        double err = abs(pi_h_u-uh).max();
        cerr << "err " << err << endl;
        return err > 1e-7 ? 1 : 0;
}
