#ifndef _COURS_QUATRATURE_CHECK_H
#define _COURS_QUATRATURE_CHECK_H
//
// check that
//      I(n) = int_{-1}^{1} x^n dx
//           = 0        if n is odd
//             2/(n+1)  if n is even
//
// is exact for 0 <= n <= order
//
#include <iterator>
#include <cmath>

template <typename Iterator1, typename Iterator2, typename Size>
typename std::iterator_traits<Iterator1>::value_type
quadrature_check (Iterator1 x, Iterator2 w, Size N, Size order) {
        typedef typename std::iterator_traits<Iterator1>::value_type T;
	if (N == 0) return 0;
        T error = 0;
        for (Size n = 0; n <= order; n++) {
            T value = 0;
            for (size_t k = 0; k <= N; k++)
                value += std::pow (x[k], int(n))*w[k];
            T exact = (n % 2 == 1) ? 0 : 2/T(n+1);
            error = std::max(error, std::fabs (exact - value));
        }
        return error;
}
#endif // _COURS_QUATRATURE_CHECK_H
