#include "sinft.h"
#include "range.h"
#include "numeric_constant.h"
#include <iostream>
template <typename Array>
std::valarray<typename Array::value_type>
sin_tranformation (const Array& u) {	
	return sin_tranformation (std::valarray<typename Array::value_type>(u));
}
#ifdef TO_CLEAN
#endif // TO_CLEAN
template <typename T>
std::valarray<T>
sin_tranformation (const std::valarray<T>& u) {	
        size_t n = u.size();
        size_t n1 = n + 1;
	size_t rest = n1 - size_t (pow (2.0, floor (log2 (1.0*n1))));
	if (rest == 0) {
	    std::valarray<T> v (n1);
	    v[0] = 0;
	    v [range(1,n)] = u;
	    sinft (&v[0]-1, n1);
	    return v [range(1,n)];
	}
	std::cerr << "warning(n != 2^p-1): using inefficient sinus transform for n = " << n << std::endl;
        const T pi = numeric_constant<T>::pi ();
        std::valarray<T> v (n);
        for (size_t i = 0; i < n; i++) {
            T sum = 0;
            for (size_t j = 0; j < n; j++)
              sum += sin ((i+1)*(j+1)*pi/(n+1)) * u [j];
            v [i] = sum;
        }
	return v;
}
