#ifndef _COURS_SMART_POINTER_H
#define _COURS_SMART_POINTER_H
//
// smart pointers
//
// author: Pierre.Saramito@imag.fr
//
// date: 27 january 2000
//
// ref: page 118:
// @book{geron-tawbi-1999,
// 	author = {A. G\'eron and F. Tawbi},
// 	title = {Pour mieux d\'evelopper avec {\tt C++}~:
// 		design pattern, {STL}, {RTTI} et
// 		smart pointers},
// 	publisher = {InterEditions},
// 	year = 1999,
// }
// 
// -----------------------------------------------------------------------
// smart_pointer
// -----------------------------------------------------------------------
template <typename T>
class smart_pointer {
public:

// allocators:

	smart_pointer (T* p = 0);
	smart_pointer (const smart_pointer&);
	~smart_pointer ();
	smart_pointer& operator= (const smart_pointer&);

// accessors:

	const T* pointer    () const;
	const T& data       () const;
	const T* operator-> () const;
	const T& operator*  () const;

// modifiers:

	T* pointer ();
	T* operator-> ();
	T& data ();
	T& operator* ();

// implementation:

private:
	struct counter {
		T*  _p;
		int _n;
		counter (T* p = 0);
		~counter ();
		int operator++ ();
		int operator-- ();
	};
	counter *_count;
};
template <typename T>
smart_pointer<T>::counter::counter (T* p)
 : _p(p), _n(1)
{
}
template <typename T>
smart_pointer<T>::counter::~counter ()
{
    delete_macro(_p);
}
template <typename T>
int
smart_pointer<T>::counter::operator++ ()
{
    return ++_n;
}
template <typename T>
int
smart_pointer<T>::counter::operator-- ()
{ 
    if (--_n != 0) return _n;
    delete_macro(this);
    return 0;
}
template <typename T>
smart_pointer<T>::smart_pointer (T* p)
: _count(new_macro(counter(p)))
{
}
template <typename T>
smart_pointer<T>::smart_pointer (const smart_pointer& sp)
 : _count(sp._count)
{
    ++(*_count);
}
template <typename T>
smart_pointer<T>::~smart_pointer ()
{
     if (_count != 0) { --(*_count); }
}
template <typename T>
smart_pointer<T>&
smart_pointer<T>::operator= (const smart_pointer& sp)
{
    if (_count != sp._count) { 
	--(*_count);
	_count = sp._count;
	++(*_count);
    }
    return *this;
}
template <typename T>
const T*
smart_pointer<T>::pointer () const
{
    return _count -> _p;
}
template <typename T>
const T&
smart_pointer<T>::data () const
{
    return *pointer();
}
template <typename T>
const T*
smart_pointer<T>::operator-> () const
{
    return pointer();
}
template <typename T>
const T&
smart_pointer<T>::operator* () const
{
    return data();
}
template <typename T>
T*
smart_pointer<T>::pointer ()
{
    if (_count -> _p == 0) return 0;
    if (_count -> _n > 1) {
   	--(_count-> _n);
	T* q = new_macro (T(*(_count -> _p)));
       	_count = new_macro (counter(q));
    }	
    return _count -> _p;
}
template <typename T>
T*
smart_pointer<T>::operator-> ()
{
     return pointer();
}
template <typename T>
T&
smart_pointer<T>::data ()
{
    return *pointer();
}
template <typename T>
T&
smart_pointer<T>::operator* ()
{
    return data();
}
#endif // _COURS_SMART_POINTER_H
